@@ -62,12 +62,13 @@ DEFAULT_SETTING_AFFINE_COUPLING = MetaDictSetting(
         "dense_args": dict(
             units=128,
             activation="relu",
-            kernel_regularizer=tf.keras.regularizers.l2(1e-4),
+            # example input: tf.keras.regularizers.l2(1e-4),
+            kernel_regularizer=None,
         ),
         "num_dense": 2,
         "spec_norm": False,
         "mc_dropout": False,
-        "dropout": True,
+        "dropout": False,
         "residual": False,
         "dropout_prob": 0.05,
         "soft_clamping": 1.9,
